@@ -4,16 +4,20 @@
 
 from os import path
 import requests
-import ast
 import time
 import re
 import copy
 import json
+from io import BytesIO
+
+from typing import Union
 
 from trafficgenerator.tgn_utils import TgnError
 
+requests.packages.urllib3.disable_warnings(requests.packages.urllib3.exceptions.InsecureRequestWarning)
 
-class IxnRestWrapper(object):
+
+class IxnRestWrapper:
 
     null = 'null'
 
@@ -42,10 +46,9 @@ class IxnRestWrapper(object):
         # Perform get to determine whether http is supported or we should use https.
         try:
             self.server_url = 'https://{}:{}'.format(ip, port)
-            a = self.get(self.server_url + '/api/v1/sessions', timeout=4)
-        except Exception as _:
-            b = self.server_url = 'http://{}:{}'.format(ip, port)
-            print(b)
+            self.get(self.server_url + '/api/v1/sessions', timeout=4)
+        except Exception as e:
+            self.server_url = 'http://{}:{}'.format(ip, port)
 
         response = self.post(self.server_url + '/api/v1/sessions', data={'applicationType': 'ixnrest'})
         if 'id' in response.json():
@@ -75,10 +78,10 @@ class IxnRestWrapper(object):
         if 'errors' in response.json():
             raise TgnError(response.json()['errors'][0])
         if response.json()['state'].lower() == 'error':
-            result = ast.literal_eval(response.content.replace('null', '""'))['result'].strip()
-            raise TgnError('wait for post {} failed - {}'.format(response.url, result))
+            result = json.loads(response.content)['result'].strip()
+            raise TgnError(f'wait for post {response.url} failed - {result}')
         if response.json()['state'].lower() == 'success':
-            return
+            return response
         progress_url = json.loads(response.content)[u'url']
         if 'http' not in progress_url:
             # sometimes the progress url is relative, for example in version 8.50 linux loadconfig
@@ -90,7 +93,7 @@ class IxnRestWrapper(object):
             else:
                 state = response.json()[0]['state']
             if state.lower() not in ['in_progress', 'down']:
-                return
+                return response
             time.sleep(1)
         raise TgnError('{} operation failed, state is {} after {} seconds'.
                        format(self.session, response.json()['state'], timeout))
@@ -119,8 +122,9 @@ class IxnRestWrapper(object):
                     error = json.loads(response.text).get('errors', None)
                 if not error:
                     error = json.loads(response.text).get('Message', None)
-            raise TgnError('failed to {} {} {}\nstatus code - {}\nerror - {}'.
-                           format(command.__name__, url, kwargs_to_print, response.status_code, error))
+            raise TgnError(f'failed to {command.__name__} {url} {kwargs_to_print}\n'
+                           f'status code - {response.status_code}\n'
+                           f'error - {error}')
         return response
 
     def get(self, url, **kwargs):
@@ -129,7 +133,7 @@ class IxnRestWrapper(object):
     def post(self, url, data={}):
         response = self.request(requests.post, url, data=data)
         if response.status_code != 201 and 'id' in response.json():
-            self.waitForComplete(url, response)
+            return self.waitForComplete(url, response)
         return response
 
     def options(self, url):
@@ -196,17 +200,24 @@ class IxnRestWrapper(object):
 
     def saveConfig(self, config_file_name):
         basename = path.basename(config_file_name)
-
-        data = {'arg1': basename}
+        data = {'arg1': path.basename(config_file_name)}
         self.post(self.root_url + 'ixnetwork/operations/saveconfig', data)
+        self.getFile(basename, config_file_name)
+
+    def getFile(self, remote_file_path, local_file_path: Union[str, BytesIO]):
 
         download_headers = {'content-type': 'application/octet-stream'}
         download_url = self.root_url + 'ixnetwork/files'
-        download_params = {'filename': basename}
+        download_params = {'filename': path.basename(remote_file_path)}
+        if path.dirname(remote_file_path):
+            download_params['absolute'] = path.dirname(remote_file_path)
         res = self.request(requests.get, download_url, headers=download_headers, params=download_params)
 
-        with open(config_file_name, mode='wb') as f:
-            f.write(res.content)
+        if type(local_file_path) is str:
+            with open(local_file_path, mode='wb') as f:
+                f.write(res.content)
+        else:
+            local_file_path.write(res.content)
 
     def getList(self, obj_ref, childList):
         response = self.get(self.server_url + obj_ref + '/' + childList)
@@ -246,10 +257,10 @@ class IxnRestWrapper(object):
     def add(self, parent, obj_type, **attributes):
         """ IXN API add command
 
-        @param parent: object parent - object will be created under this parent.
-        @param object_type: object type.
-        @param attributes: additional attributes.
-        @return: STC object reference.
+        :param parent: object parent - object will be created under this parent.
+        :param object_type: object type.
+        :param attributes: additional attributes.
+        @return: IXN object reference.
         """
 
         response = self.post(self.server_url + parent.ref + '/' + obj_type, attributes)
