@@ -1,71 +1,65 @@
 """
 Classes and utilities to manage IXN application.
-
-@author yoram@ignissoft.com
 """
 
 from os import path
 import time
 from collections import OrderedDict
+from logging import Logger
+
+from typing import Type, Optional, Union, Dict, List
 
 from trafficgenerator.tgn_utils import TgnError, is_true, ApiType
 from trafficgenerator.tgn_app import TgnApp
 
+from ixnetwork import TYPE_2_OBJECT
 from ixnetwork.api.ixn_tcl import IxnTclWrapper
-from ixnetwork.api.ixn_python import IxnPythonWrapper
 from ixnetwork.api.ixn_rest import IxnRestWrapper
 from ixnetwork.ixn_object import IxnObject
-from ixnetwork.ixn_port import IxnPort
-from ixnetwork.ixn_traffic import IxnTrafficItem
-from ixnetwork.ixn_protocol import (IxnBgpRouter, IxnOspfRouter, IxnOspfV3Router, IxnBgpRouteRange, IxnOspfRouteRange,
-                                    IxnPimsmRouter, IxnIgmpHost, IxnIgmpQuerier, IxnPimsmSource, IxnStpBridge,
-                                    IxnOpenFlowDevice, IxnLacp, IxnIsisRouter, IxnIsisRouteRange, IxnLdpRouter,
-                                    IxnLdpRouteRange, IxnRsvpNeighborPair, IxnStaticIp, IxnBfdfRouter,
-                                    IxnOspfv3RouteRange)
-from ixnetwork.ixn_interface import IxnInterface, IxnInterfaceL3
-from ixnetwork.ixn_topology import (IxnTopology, IxnNgpfEthernet, IxnNgpfVlan, IxnNgpfIpv4, IxnNgpfIpv6,
-                                    IxnDeviceGroup, IxnNetworkGroup)
 from ixnetwork.ixn_root import IxnRoot
-from ixnetwork.ixn_protocol_stack import IxnRange
-from ixnetwork.ixn_hw import IxnHw, IxnChassis, IxnCard, IxnPhyPort
+from ixnetwork.ixn_port import IxnPort
 
 
-def init_ixn(api, logger, install_dir=None):
+def init_ixn(api: ApiType, logger: Type[Logger], install_dir: Optional[str] = None) -> 'IxnApp':
     """ Create IXN object.
 
-    :param api: tcl/python/rest
-    :type api: trafficgenerator.tgn_utils.ApiType
-    :param logger: logger object
-    :param install_dir: IXN installation directory
+    :param TgnApp api: tcl/rest
+    :param Logger logger: logger object
+    :param str install_dir: IXN installation directory, applicable for Tcl only
     :return: IXN object
     """
 
     if api == ApiType.tcl:
         api_wrapper = IxnTclWrapper(logger, install_dir)
-    elif api == ApiType.python:
-        api_wrapper = IxnPythonWrapper(logger, install_dir)
     elif api == ApiType.rest:
         api_wrapper = IxnRestWrapper(logger)
     else:
-        raise TgnError('{} API not supported - use Tcl, python or REST'.format(api))
+        raise TgnError(f'{api} API not supported - use {ApiType.tcl} or {ApiType.rest}')
     return IxnApp(logger, api_wrapper)
 
 
 class IxnApp(TgnApp):
     """ IxNetwork driver. Equivalent to IxNetwork Application. """
 
-    def __init__(self, logger, api_wrapper):
+    def __init__(self, logger: Type[Logger], api_wrapper: Union[IxnTclWrapper, IxnRestWrapper]) -> None:
         """ Set all kinds of application level objects - logger, api, etc.
 
         :param logger: python logger (e.g. logging.getLogger('log'))
-        :param api_wrapper: api wrapper object inheriting and implementing IxnApi base class.
+        :param api_wrapper: api wrapper object inheriting
         """
-
         super(self.__class__, self).__init__(logger, api_wrapper)
-
         IxnObject.str_2_class = TYPE_2_OBJECT
 
-    def connect(self, api_server='localhost', api_port=11009, auth=None):
+        self.root = None
+
+    def connect(self, api_server: str = 'localhost', api_port: Optional[int] = 11009,
+                auth: Optional[List[str]] = None) -> None:
+        """ Connect to API server.
+
+        :param api_server: API server IP address
+        :param api_port: API server TCP port
+        :param auth: list of (username, password)
+        """
         self.api.connect(api_server, api_port, auth)
         self.root = IxnRoot(objRef=self.api.getRoot(), objType='root', parent=None)
         self.root.logger = self.logger
@@ -73,8 +67,8 @@ class IxnApp(TgnApp):
         IxnObject.root = self.root
         self.root.hw = self.root.get_child_static('availableHardware')
 
-    def disconnect(self):
-        """ Disconnect from chassis and server. """
+    def disconnect(self) -> None:
+        """ Disconnect from API server. """
         if self.root.ref is not None:
             self.api.disconnect()
         self.root = None
@@ -83,10 +77,15 @@ class IxnApp(TgnApp):
     # IxNetwork operation commands.
     #
 
-    def commit(self):
+    def commit(self) -> None:
+        """ Perform ixNet commit command, NA (pass) for rest API. """
         self.api.commit()
 
-    def load_config(self, config_file_name):
+    def load_config(self, config_file_name: str) -> None:
+        """ Load ixncgf configuration file.
+
+        :param config_file_name: full path to ixncfg configuration file
+        """
         self.root.objects = OrderedDict()
         prefs = self.root.get_child_static('globals').get_child_static('preferences')
         prefs.set_attributes(connectPortsOnLoadConfig=False)
@@ -95,12 +94,17 @@ class IxnApp(TgnApp):
         self.root.get_children('vport')
         self.root.hw = self.root.get_child_static('availableHardware')
 
-    def new_config(self):
+    def new_config(self) -> None:
+        """ Clean existing configuration and creates empty one. """
         self.root.objects = OrderedDict()
         self.api.newConfig()
         self.commit()
 
-    def save_config(self, config_file_name):
+    def save_config(self, config_file_name: str) -> None:
+        """ Save ixncgf configuration file.
+
+        :param config_file_name: full path to ixncfg configuration file
+        """
         self.commit()
         self.api.saveConfig(path.abspath(config_file_name))
 
@@ -108,7 +112,7 @@ class IxnApp(TgnApp):
     # IxNetwork GUI commands.
     #
 
-    def reserve(self, ports, force=False, wait_for_up=True, timeout=80):
+    def reserve(self, ports: Dict[IxnPort, str], force=False, wait_for_up=True, timeout=80) -> None:
         """ Reserve port and optionally wait for port to come up.
 
         :param ports: dict of <port, ip/module/port'>.
@@ -124,21 +128,19 @@ class IxnApp(TgnApp):
         for port, location in ports.items():
             port.reserve(location, False, wait_for_up, timeout)
 
-    def send_arp_ns(self):
+    def send_arp_ns(self) -> None:
+        """ Perform ixNet sendArpAll/sendNsAll Tcl commands. NA (pass) for rest API. """
         self.api.execute('sendArpAll', valid_on_linux=False)
         self.api.execute('sendNsAll', valid_on_linux=False)
 
-    def send_rs(self):
-        self.api.execute('sendRsAll', valid_on_linux=False)
-
-    def protocols_start(self):
+    def protocols_start(self) -> None:
         """ Start all protocols.
 
         It is the calling function responsibility to wait for all protocols to start.
         """
         self.api.execute('startAllProtocols')
 
-    def protocols_stop(self):
+    def protocols_stop(self) -> None:
         """ Stop all protocols.
 
         It is the calling function responsibility to wait for all protocols to stop.
@@ -198,65 +200,10 @@ class IxnApp(TgnApp):
                                format(action, port.obj_name(), protocol))
 
     def quick_test_apply(self, name):
-        quick_test = self.root.get_quick_tests()[name]
-        quick_test.execute('apply', quick_test.ref)
+        self.root.quick_tests[name].apply()
 
     def quick_test_start(self, name, blocking=False, timeout=3600):
-        quick_test = self.root.get_quick_tests()[name]
-        quick_test.execute('start', quick_test.ref)
-        if blocking:
-            return self.wait_quick_test_status(name, False, timeout)
+        return self.root.quick_tests[name].start(blocking, timeout)
 
     def quick_test_stop(self, name):
-        quick_test = self.root.get_quick_tests()[name]
-        quick_test.execute('stop', quick_test.ref)
-
-    def wait_quick_test_status(self, name, status=False, timeout=3600):
-        quick_test = self.root.get_quick_tests()[name]
-        results = quick_test.get_child_static('results')
-        for _ in range(timeout):
-            if is_true(results.get_attribute('isRunning')) == status:
-                return results.get_attribute('result')
-            time.sleep(1)
-        raise TgnError('Quick test failed, quick test running state is {} after {} seconds'.
-                       format(results.get_attribute('isRunning'), timeout))
-
-
-TYPE_2_OBJECT = {'availableHardware': IxnHw,
-                 'bridge': IxnStpBridge,
-                 'card': IxnCard,
-                 'chassis': IxnChassis,
-                 'device': IxnOpenFlowDevice,
-                 'deviceGroup': IxnDeviceGroup,
-                 'ethernet': IxnNgpfEthernet,
-                 'host': IxnIgmpHost,
-                 'interface': {'vport': IxnInterface},
-                 'ip': IxnStaticIp,
-                 'ipv4': {'interface': IxnInterfaceL3,
-                          'etherenet': IxnNgpfIpv4},
-                 'ipv6': {'interface': IxnInterfaceL3,
-                          'etherenet': IxnNgpfIpv6},
-                 'lacp': IxnLacp,
-                 'neighborPair': IxnRsvpNeighborPair,
-                 'neighborRange': IxnBgpRouter,
-                 'networkGroup': IxnNetworkGroup,
-                 'port': IxnPhyPort,
-                 'querier': IxnIgmpQuerier,
-                 'range': IxnRange,
-                 'router': {'bfd': IxnBfdfRouter,
-                            'isis': IxnIsisRouter,
-                            'ldp': IxnLdpRouter,
-                            'ospf': IxnOspfRouter,
-                            'ospfV3': IxnOspfV3Router,
-                            'pimsm': IxnPimsmRouter},
-                 'routeRange': {'bgp': IxnBgpRouteRange,
-                                'ospf': IxnOspfRouteRange,
-                                'ospfV3': IxnOspfv3RouteRange,
-                                'isis': IxnIsisRouteRange,
-                                'ldp': IxnLdpRouteRange},
-                 'source': {'interface': IxnPimsmSource},
-                 'topology': IxnTopology,
-                 'trafficItem': IxnTrafficItem,
-                 'vlan': IxnNgpfVlan,
-                 'vport': IxnPort,
-                 }
+        return self.root.quick_tests[name].stop()
